@@ -19,39 +19,36 @@
 #ifndef TESSERACT_CCUTIL_GENERICVECTOR_H_
 #define TESSERACT_CCUTIL_GENERICVECTOR_H_
 
-#include <algorithm>
-#include <cassert>
-#include <climits>      // for LONG_MAX
-#include <cstdint>      // for uint32_t
-#include <cstdio>
-#include <cstdlib>
-
 #include "helpers.h"
 #include "serialis.h"
-#include "strngs.h"
-#include "tesscallback.h"
+
+#include <algorithm>
+#include <cassert>
+#include <climits> // for LONG_MAX
+#include <cstdint> // for uint32_t
+#include <cstdio>
+#include <cstdlib>
+#include <functional> // for std::function
+
+namespace tesseract {
 
 // Use PointerVector<T> below in preference to GenericVector<T*>, as that
 // provides automatic deletion of pointers, [De]Serialize that works, and
 // sort that works.
 template <typename T>
 class GenericVector {
- public:
+public:
   GenericVector() {
     init(kDefaultVectorSize);
   }
-  GenericVector(int size, const T& init_val) {
-    init(size);
-    init_to_size(size, init_val);
-  }
 
   // Copy
-  GenericVector(const GenericVector& other) {
+  GenericVector(const GenericVector &other) {
     this->init(other.size());
     this->operator+=(other);
   }
-  GenericVector<T>& operator+=(const GenericVector& other);
-  GenericVector<T>& operator=(const GenericVector& other);
+  GenericVector<T> &operator+=(const GenericVector &other);
+  GenericVector<T> &operator=(const GenericVector &other);
 
   ~GenericVector();
 
@@ -61,7 +58,8 @@ class GenericVector {
   void double_the_size();
 
   // Resizes to size and sets all values to t.
-  void init_to_size(int size, const T& t);
+  void init_to_size(int size, const T &t);
+  void resize(int size, const T &t);
   // Resizes to size without any initialization.
   void resize_no_init(int size) {
     reserve(size);
@@ -69,13 +67,12 @@ class GenericVector {
   }
 
   // Return the size used.
-  int size() const {
+  unsigned size() const {
     return size_used_;
   }
   // Workaround to avoid g++ -Wsign-compare warnings.
   size_t unsigned_size() const {
-    static_assert(sizeof(size_used_) <= sizeof(size_t),
-                  "Wow! sizeof(size_t) < sizeof(int32_t)!!");
+    static_assert(sizeof(size_used_) <= sizeof(size_t), "Wow! sizeof(size_t) < sizeof(int32_t)!!");
     assert(0 <= size_used_);
     return static_cast<size_t>(size_used_);
   }
@@ -83,50 +80,34 @@ class GenericVector {
     return size_reserved_;
   }
 
-  int length() const {
-    return size_used_;
-  }
-
   // Return true if empty.
   bool empty() const {
     return size_used_ == 0;
   }
 
   // Return the object from an index.
-  T& get(int index) const;
-  T& back() const;
-  T& operator[](int index) const;
+  T &at(int index) const {
+    assert(index >= 0 && index < size_used_);
+    return data_[index];
+  }
+
+  T &back() const;
+  T &operator[](int index) const;
   // Returns the last object and removes it.
   T pop_back();
 
   // Return the index of the T object.
-  // This method NEEDS a compare_callback to be passed to
-  // set_compare_callback.
-  int get_index(const T& object) const;
-
-  // Return true if T is in the array
-  bool contains(const T& object) const;
-
-  // Return true if the index is valid
-  T contains_index(int index) const;
+  int get_index(const T &object) const;
 
   // Push an element in the end of the array
   int push_back(T object);
-  void operator+=(const T& t);
-
-  // Push an element in the end of the array if the same
-  // element is not already contained in the array.
-  int push_back_new(const T& object);
-
-  // Push an element in the front of the array
-  // Note: This function is O(n)
-  int push_front(const T& object);
+  void operator+=(const T &t);
 
   // Set the value at the given index
-  void set(const T& t, int index);
+  void set(const T &t, int index);
 
   // Insert t at the given index, push other elements to the right.
-  void insert(const T& t, int index);
+  void insert(const T &t, int index);
 
   // Removes an element at the given index and
   // shifts the remaining elements to the left.
@@ -142,16 +123,10 @@ class GenericVector {
 
   // Add a callback to be called to delete the elements when the array took
   // their ownership.
-  void set_clear_callback(TessCallback1<T>* cb) {
+  void set_clear_callback(std::function<void(T)> cb) {
     clear_cb_ = cb;
   }
 
-  // Add a callback to be called to compare the elements when needed (contains,
-  // get_id, ...)
-  void set_compare_callback(TessResultCallback2<bool, T const&, T const&>* cb) {
-    compare_cb_ = cb;
-  }
-
   // Clear the array, calling the clear callback function if any.
   // All the owned callbacks are also deleted.
   // If you don't want the callbacks to be deleted, before calling clear, set
@@ -164,7 +139,7 @@ class GenericVector {
   // This method clears the current object, then, does a shallow copy of
   // its argument, and finally invalidates its argument.
   // Callbacks are moved to the current object;
-  void move(GenericVector<T>* from);
+  void move(GenericVector<T> *from);
 
   // Read/Write the array to a file. This does _NOT_ read/write the callbacks.
   // The callback given must be permanent since they will be called more than
@@ -173,54 +148,35 @@ class GenericVector {
   // fread (and swapping)/fwrite.
   // Returns false on error or if the callback returns false.
   // DEPRECATED. Use [De]Serialize[Classes] instead.
-  bool write(FILE* f, TessResultCallback2<bool, FILE*, T const&>* cb) const;
-  bool read(tesseract::TFile* f,
-            TessResultCallback2<bool, tesseract::TFile*, T*>* cb);
+  bool write(FILE *f, std::function<bool(FILE *, const T &)> cb) const;
+  bool read(TFile *f, std::function<bool(TFile *, T *)> cb);
   // Writes a vector of simple types to the given file. Assumes that bitwise
   // read/write of T will work. Returns false in case of error.
   // TODO(rays) Change all callers to use TFile and remove deprecated methods.
-  bool Serialize(FILE* fp) const;
-  bool Serialize(tesseract::TFile* fp) const;
+  bool Serialize(FILE *fp) const;
+  bool Serialize(TFile *fp) const;
   // Reads a vector of simple types from the given file. Assumes that bitwise
   // read/write will work with ReverseN according to sizeof(T).
   // Returns false in case of error.
   // If swap is true, assumes a big/little-endian swap is needed.
   // TFile is assumed to know about swapping.
-  bool DeSerialize(bool swap, FILE* fp);
-  bool DeSerialize(tesseract::TFile* fp);
-  // Skips the deserialization of the vector.
-  static bool SkipDeSerialize(tesseract::TFile* fp);
+  bool DeSerialize(bool swap, FILE *fp);
+  bool DeSerialize(TFile *fp);
   // Writes a vector of classes to the given file. Assumes the existence of
   // bool T::Serialize(FILE* fp) const that returns false in case of error.
   // Returns false in case of error.
-  bool SerializeClasses(FILE* fp) const;
-  bool SerializeClasses(tesseract::TFile* fp) const;
+  bool SerializeClasses(FILE *fp) const;
   // Reads a vector of classes from the given file. Assumes the existence of
   // bool T::Deserialize(bool swap, FILE* fp) that returns false in case of
   // error. Also needs T::T() and T::T(constT&), as init_to_size is used in
   // this function. Returns false in case of error.
   // If swap is true, assumes a big/little-endian swap is needed.
-  bool DeSerializeClasses(bool swap, FILE* fp);
-  bool DeSerializeClasses(tesseract::TFile* fp);
-  // Calls SkipDeSerialize on the elements of the vector.
-  static bool SkipDeSerializeClasses(tesseract::TFile* fp);
-
-  // Allocates a new array of double the current_size, copies over the
-  // information from data to the new location, deletes data and returns
-  // the pointed to the new larger array.
-  // This function uses memcpy to copy the data, instead of invoking
-  // operator=() for each element like double_the_size() does.
-  static T* double_the_size_memcpy(int current_size, T* data) {
-    T* data_new = new T[current_size * 2];
-    memcpy(data_new, data, sizeof(T) * current_size);
-    delete[] data;
-    return data_new;
-  }
+  bool DeSerializeClasses(TFile *fp);
 
   // Reverses the elements of the vector.
   void reverse() {
     for (int i = 0; i < size_used_ / 2; ++i) {
-      Swap(&data_[i], &data_[size_used_ - 1 - i]);
+      std::swap(data_[i], data_[size_used_ - 1 - i]);
     }
   }
 
@@ -235,101 +191,10 @@ class GenericVector {
   // The comparator function is as defined by qsort, ie. it receives pointers
   // to two Ts and returns negative if the first element is to appear earlier
   // in the result and positive if it is to appear later, with 0 for equal.
-  void sort(int (*comparator)(const void*, const void*)) {
+  void sort(int (*comparator)(const void *, const void *)) {
     qsort(data_, size_used_, sizeof(*data_), comparator);
   }
 
-  // Searches the array (assuming sorted in ascending order, using sort()) for
-  // an element equal to target and returns true if it is present.
-  // Use binary_search to get the index of target, or its nearest candidate.
-  bool bool_binary_search(const T& target) const {
-    int index = binary_search(target);
-    if (index >= size_used_) {
-      return false;
-    }
-    return data_[index] == target;
-  }
-  // Searches the array (assuming sorted in ascending order, using sort()) for
-  // an element equal to target and returns the index of the best candidate.
-  // The return value is conceptually the largest index i such that
-  // data_[i] <= target or 0 if target < the whole vector.
-  // NOTE that this function uses operator> so really the return value is
-  // the largest index i such that data_[i] > target is false.
-  int binary_search(const T& target) const {
-    int bottom = 0;
-    int top = size_used_;
-    while (top - bottom > 1) {
-      int middle = (bottom + top) / 2;
-      if (data_[middle] > target) {
-        top = middle;
-      } else {
-        bottom = middle;
-      }
-    }
-    return bottom;
-  }
-
-  // Compact the vector by deleting elements using operator!= on basic types.
-  // The vector must be sorted.
-  void compact_sorted() {
-    if (size_used_ == 0) {
-      return;
-    }
-
-    // First element is in no matter what, hence the i = 1.
-    int last_write = 0;
-    for (int i = 1; i < size_used_; ++i) {
-      // Finds next unique item and writes it.
-      if (data_[last_write] != data_[i]) {
-        data_[++last_write] = data_[i];
-      }
-    }
-    // last_write is the index of a valid data cell, so add 1.
-    size_used_ = last_write + 1;
-  }
-
-  // Compact the vector by deleting elements for which delete_cb returns
-  // true. delete_cb is a permanent callback and will be deleted.
-  void compact(TessResultCallback1<bool, int>* delete_cb) {
-    int new_size = 0;
-    int old_index = 0;
-    // Until the callback returns true, the elements stay the same.
-    while (old_index < size_used_ && !delete_cb->Run(old_index++)) {
-      ++new_size;
-    }
-    // Now just copy anything else that gets false from delete_cb.
-    for (; old_index < size_used_; ++old_index) {
-      if (!delete_cb->Run(old_index)) {
-        data_[new_size++] = data_[old_index];
-      }
-    }
-    size_used_ = new_size;
-    delete delete_cb;
-  }
-
-  T dot_product(const GenericVector<T>& other) const {
-    T result = static_cast<T>(0);
-    for (int i = std::min(size_used_, other.size_used_) - 1; i >= 0; --i) {
-      result += data_[i] * other.data_[i];
-    }
-    return result;
-  }
-
-  // Returns the index of what would be the target_index_th item in the array
-  // if the members were sorted, without actually sorting. Members are
-  // shuffled around, but it takes O(n) time.
-  // NOTE: uses operator< and operator== on the members.
-  int choose_nth_item(int target_index) {
-    // Make sure target_index is legal.
-    if (target_index < 0) {
-      target_index = 0;  // ensure legal
-    } else if (target_index >= size_used_) {
-      target_index = size_used_ - 1;
-    }
-    unsigned int seed = 1;
-    return choose_nth_item(target_index, 0, size_used_, &seed);
-  }
-
   // Swaps the elements with the given indices.
   void swap(int index1, int index2) {
     if (index1 != index2) {
@@ -340,19 +205,16 @@ class GenericVector {
   }
   // Returns true if all elements of *this are within the given range.
   // Only uses operator<
-  bool WithinBounds(const T& rangemin, const T& rangemax) const {
-    for (int i = 0; i < size_used_; ++i) {
-      if (data_[i] < rangemin || rangemax < data_[i]) {
-        return false;
-      }
+  /*bool WithinBounds(const T& rangemin, const T& rangemax) const {
+  for (int i = 0; i < size_used_; ++i) {
+    if (data_[i] < rangemin || rangemax < data_[i]) {
+      return false;
     }
-    return true;
   }
+  return true;
+}*/
 
- protected:
-  // Internal recursive version of choose_nth_item.
-  int choose_nth_item(int target_index, int start, int end, unsigned int* seed);
-
+protected:
   // Init the object, allocating size memory.
   void init(int size);
 
@@ -362,19 +224,15 @@ class GenericVector {
   static const int kDefaultVectorSize = 4;
   int32_t size_used_{};
   int32_t size_reserved_{};
-  T* data_;
-  TessCallback1<T>* clear_cb_;
-  // Mutable because Run method is not const
-  mutable TessResultCallback2<bool, T const&, T const&>* compare_cb_;
+  T *data_;
+  std::function<void(T)> clear_cb_;
 };
 
-namespace tesseract {
-
 // The default FileReader loads the whole file into the vector of char,
 // returning false on error.
-inline bool LoadDataFromFile(const char* filename, GenericVector<char>* data) {
+inline bool LoadDataFromFile(const char *filename, GenericVector<char> *data) {
   bool result = false;
-  FILE* fp = fopen(filename, "rb");
+  FILE *fp = fopen(filename, "rb");
   if (fp != nullptr) {
     fseek(fp, 0, SEEK_END);
     auto size = std::ftell(fp);
@@ -384,6 +242,7 @@ inline bool LoadDataFromFile(const char* filename, GenericVector<char>* data) {
       // reserve an extra byte in case caller wants to append a '\0' character
       data->reserve(size + 1);
       data->resize_no_init(size);
+
     /* willus mod Dec 2018--weird issue with Win XP and MinGW gcc 7.3.0 */
     /* Can't read entire file at once -- need to break up into smaller blocksize reads */
     {
@@ -401,47 +260,35 @@ inline bool LoadDataFromFile(const char* filename, GenericVector<char>* data) {
         }
     result = static_cast<long>((long)n==size);
     }
-    /*
-    result = static_cast<long>(fread(&(*data)[0], 1, size, fp)) == size;
-    */
+      /*
+      result = static_cast<long>(fread(&(*data)[0], 1, size, fp)) == size;
+      */
     }
     fclose(fp);
   }
   return result;
 }
 
-inline bool LoadDataFromFile(const STRING& filename,
-                             GenericVector<char>* data) {
-  return LoadDataFromFile(filename.string(), data);
-}
-
 // The default FileWriter writes the vector of char to the filename file,
 // returning false on error.
-inline bool SaveDataToFile(const GenericVector<char>& data,
-                           const STRING& filename) {
-  FILE* fp = fopen(filename.string(), "wb");
+inline bool SaveDataToFile(const GenericVector<char> &data, const char *filename) {
+  FILE *fp = fopen(filename, "wb");
   if (fp == nullptr) {
     return false;
   }
-  bool result =
-      static_cast<int>(fwrite(&data[0], 1, data.size(), fp)) == data.size();
+  bool result = fwrite(&data[0], 1, data.size(), fp) == data.size();
   fclose(fp);
   return result;
 }
 
-template <typename T>
-bool cmp_eq(T const& t1, T const& t2) {
-  return t1 == t2;
-}
-
 // Used by sort()
 // return < 0 if t1 < t2
 // return 0 if t1 == t2
 // return > 0 if t1 > t2
 template <typename T>
-int sort_cmp(const void* t1, const void* t2) {
-  const T* a = static_cast<const T*>(t1);
-  const T* b = static_cast<const T*>(t2);
+int sort_cmp(const void *t1, const void *t2) {
+  const T *a = static_cast<const T *>(t1);
+  const T *b = static_cast<const T *>(t2);
   if (*a < *b) {
     return -1;
   }
@@ -456,9 +303,9 @@ int sort_cmp(const void* t1, const void* t2) {
 // return 0 if t1 == t2
 // return > 0 if t1 > t2
 template <typename T>
-int sort_ptr_cmp(const void* t1, const void* t2) {
-  const T* a = *static_cast<T* const*>(t1);
-  const T* b = *static_cast<T* const*>(t2);
+int sort_ptr_cmp(const void *t1, const void *t2) {
+  const T *a = *static_cast<T *const *>(t1);
+  const T *b = *static_cast<T *const *>(t2);
   if (*a < *b) {
     return -1;
   }
@@ -472,10 +319,10 @@ int sort_ptr_cmp(const void* t1, const void* t2) {
 // as it provides automatic deletion and correct serialization, with the
 // corollary that all copy operations are deep copies of the pointed-to objects.
 template <typename T>
-class PointerVector : public GenericVector<T*> {
- public:
-  PointerVector() : GenericVector<T*>() {}
-  explicit PointerVector(int size) : GenericVector<T*>(size) {}
+class PointerVector : public GenericVector<T *> {
+public:
+  PointerVector() : GenericVector<T *>() {}
+  explicit PointerVector(int size) : GenericVector<T *>(size) {}
   ~PointerVector() {
     // Clear must be called here, even though it is called again by the base,
     // as the base will call the wrong clear.
@@ -483,19 +330,19 @@ class PointerVector : public GenericVector<T*> {
   }
   // Copy must be deep, as the pointers will be automatically deleted on
   // destruction.
-  PointerVector(const PointerVector& other) : GenericVector<T*>(other) {
+  PointerVector(const PointerVector &other) : GenericVector<T *>(other) {
     this->init(other.size());
     this->operator+=(other);
   }
-  PointerVector<T>& operator+=(const PointerVector& other) {
+  PointerVector<T> &operator+=(const PointerVector &other) {
     this->reserve(this->size_used_ + other.size_used_);
-    for (int i = 0; i < other.size(); ++i) {
+    for (unsigned i = 0; i < other.size(); ++i) {
       this->push_back(new T(*other.data_[i]));
     }
     return *this;
   }
 
-  PointerVector<T>& operator=(const PointerVector& other) {
+  PointerVector<T> &operator=(const PointerVector &other) {
     if (&other != this) {
       this->truncate(0);
       this->operator+=(other);
@@ -506,40 +353,17 @@ class PointerVector : public GenericVector<T*> {
   // Removes an element at the given index and
   // shifts the remaining elements to the left.
   void remove(int index) {
-    delete GenericVector<T*>::data_[index];
-    GenericVector<T*>::remove(index);
+    delete GenericVector<T *>::data_[index];
+    GenericVector<T *>::remove(index);
   }
 
   // Truncates the array to the given size by removing the end.
   // If the current size is less, the array is not expanded.
   void truncate(int size) {
-    for (int i = size; i < GenericVector<T*>::size_used_; ++i) {
-      delete GenericVector<T*>::data_[i];
+    for (int i = size; i < GenericVector<T *>::size_used_; ++i) {
+      delete GenericVector<T *>::data_[i];
     }
-    GenericVector<T*>::truncate(size);
-  }
-
-  // Compact the vector by deleting elements for which delete_cb returns
-  // true. delete_cb is a permanent callback and will be deleted.
-  void compact(TessResultCallback1<bool, const T*>* delete_cb) {
-    int new_size = 0;
-    int old_index = 0;
-    // Until the callback returns true, the elements stay the same.
-    while (old_index < GenericVector<T*>::size_used_ &&
-           !delete_cb->Run(GenericVector<T*>::data_[old_index++])) {
-      ++new_size;
-    }
-    // Now just copy anything else that gets false from delete_cb.
-    for (; old_index < GenericVector<T*>::size_used_; ++old_index) {
-      if (!delete_cb->Run(GenericVector<T*>::data_[old_index])) {
-        GenericVector<T*>::data_[new_size++] =
-            GenericVector<T*>::data_[old_index];
-      } else {
-        delete GenericVector<T*>::data_[old_index];
-      }
-    }
-    GenericVector<T*>::size_used_ = new_size;
-    delete delete_cb;
+    GenericVector<T *>::truncate(size);
   }
 
   // Clear the array, calling the clear callback function if any.
@@ -547,8 +371,8 @@ class PointerVector : public GenericVector<T*> {
   // If you don't want the callbacks to be deleted, before calling clear, set
   // the callback to nullptr.
   void clear() {
-    GenericVector<T*>::delete_data_pointers();
-    GenericVector<T*>::clear();
+    GenericVector<T *>::delete_data_pointers();
+    GenericVector<T *>::clear();
   }
 
   // Writes a vector of (pointers to) classes to the given file. Assumes the
@@ -556,33 +380,33 @@ class PointerVector : public GenericVector<T*> {
   // error. There is no Serialize for simple types, as you would have a
   // normal GenericVector of those.
   // Returns false in case of error.
-  bool Serialize(FILE* fp) const {
-    int32_t used = GenericVector<T*>::size_used_;
+  bool Serialize(FILE *fp) const {
+    int32_t used = GenericVector<T *>::size_used_;
     if (fwrite(&used, sizeof(used), 1, fp) != 1) {
       return false;
     }
     for (int i = 0; i < used; ++i) {
-      int8_t non_null = GenericVector<T*>::data_[i] != nullptr;
+      int8_t non_null = GenericVector<T *>::data_[i] != nullptr;
       if (fwrite(&non_null, sizeof(non_null), 1, fp) != 1) {
         return false;
       }
-      if (non_null && !GenericVector<T*>::data_[i]->Serialize(fp)) {
+      if (non_null && !GenericVector<T *>::data_[i]->Serialize(fp)) {
         return false;
       }
     }
     return true;
   }
-  bool Serialize(TFile* fp) const {
-    int32_t used = GenericVector<T*>::size_used_;
+  bool Serialize(TFile *fp) const {
+    int32_t used = GenericVector<T *>::size_used_;
     if (fp->FWrite(&used, sizeof(used), 1) != 1) {
       return false;
     }
     for (int i = 0; i < used; ++i) {
-      int8_t non_null = GenericVector<T*>::data_[i] != nullptr;
+      int8_t non_null = GenericVector<T *>::data_[i] != nullptr;
       if (fp->FWrite(&non_null, sizeof(non_null), 1) != 1) {
         return false;
       }
-      if (non_null && !GenericVector<T*>::data_[i]->Serialize(fp)) {
+      if (non_null && !GenericVector<T *>::data_[i]->Serialize(fp)) {
         return false;
       }
     }
@@ -595,7 +419,7 @@ class PointerVector : public GenericVector<T*> {
   // If swap is true, assumes a big/little-endian swap is needed.
   // Also needs T::T(), as new T is used in this function.
   // Returns false in case of error.
-  bool DeSerialize(bool swap, FILE* fp) {
+  bool DeSerialize(bool swap, FILE *fp) {
     uint32_t reserved;
     if (fread(&reserved, sizeof(reserved), 1, fp) != 1) {
       return false;
@@ -608,14 +432,14 @@ class PointerVector : public GenericVector<T*> {
     if (reserved > UINT16_MAX) {
       return false;
     }
-    GenericVector<T*>::reserve(reserved);
+    GenericVector<T *>::reserve(reserved);
     truncate(0);
     for (uint32_t i = 0; i < reserved; ++i) {
       int8_t non_null;
       if (fread(&non_null, sizeof(non_null), 1, fp) != 1) {
         return false;
       }
-      T* item = nullptr;
+      T *item = nullptr;
       if (non_null != 0) {
         item = new T;
         if (!item->DeSerialize(swap, fp)) {
@@ -630,81 +454,11 @@ class PointerVector : public GenericVector<T*> {
     }
     return true;
   }
-  bool DeSerialize(TFile* fp) {
-    int32_t reserved;
-    if (!DeSerializeSize(fp, &reserved)) {
-      return false;
-    }
-    GenericVector<T*>::reserve(reserved);
-    truncate(0);
-    for (int i = 0; i < reserved; ++i) {
-      if (!DeSerializeElement(fp)) {
-        return false;
-      }
-    }
-    return true;
-  }
-  // Enables deserialization of a selection of elements. Note that in order to
-  // retain the integrity of the stream, the caller must call some combination
-  // of DeSerializeElement and DeSerializeSkip of the exact number returned in
-  // *size, assuming a true return.
-  static bool DeSerializeSize(TFile* fp, int32_t* size) {
-    return fp->FReadEndian(size, sizeof(*size), 1) == 1;
-  }
-  // Reads and appends to the vector the next element of the serialization.
-  bool DeSerializeElement(TFile* fp) {
-    int8_t non_null;
-    if (fp->FRead(&non_null, sizeof(non_null), 1) != 1) {
-      return false;
-    }
-    T* item = nullptr;
-    if (non_null != 0) {
-      item = new T;
-      if (!item->DeSerialize(fp)) {
-        delete item;
-        return false;
-      }
-      this->push_back(item);
-    } else {
-      // Null elements should keep their place in the vector.
-      this->push_back(nullptr);
-    }
-    return true;
-  }
-  // Skips the next element of the serialization.
-  static bool DeSerializeSkip(TFile* fp) {
-    int8_t non_null;
-    if (fp->FRead(&non_null, sizeof(non_null), 1) != 1) {
-      return false;
-    }
-    if (non_null != 0) {
-      if (!T::SkipDeSerialize(fp)) {
-        return false;
-      }
-    }
-    return true;
-  }
 
   // Sorts the items pointed to by the members of this vector using
   // t::operator<().
   void sort() {
-    this->GenericVector<T*>::sort(&sort_ptr_cmp<T>);
-  }
-};
-
-}  // namespace tesseract
-
-// A useful vector that uses operator== to do comparisons.
-template <typename T>
-class GenericVectorEqEq : public GenericVector<T> {
- public:
-  GenericVectorEqEq() {
-    GenericVector<T>::set_compare_callback(
-        NewPermanentTessCallback(tesseract::cmp_eq<T>));
-  }
-  explicit GenericVectorEqEq(int size) : GenericVector<T>(size) {
-    GenericVector<T>::set_compare_callback(
-        NewPermanentTessCallback(tesseract::cmp_eq<T>));
+    this->GenericVector<T *>::sort(&sort_ptr_cmp<T>);
   }
 };
 
@@ -722,7 +476,6 @@ void GenericVector<T>::init(int size) {
     size_reserved_ = size;
   }
   clear_cb_ = nullptr;
-  compare_cb_ = nullptr;
 }
 
 template <typename T>
@@ -740,7 +493,7 @@ void GenericVector<T>::reserve(int size) {
   if (size < kDefaultVectorSize) {
     size = kDefaultVectorSize;
   }
-  T* new_array = new T[size];
+  T *new_array = new T[size];
   for (int i = 0; i < size_used_; ++i) {
     new_array[i] = data_[i];
   }
@@ -760,7 +513,7 @@ void GenericVector<T>::double_the_size() {
 
 // Resizes to size and sets all values to t.
 template <typename T>
-void GenericVector<T>::init_to_size(int size, const T& t) {
+void GenericVector<T>::init_to_size(int size, const T &t) {
   reserve(size);
   size_used_ = size;
   for (int i = 0; i < size; ++i) {
@@ -768,21 +521,19 @@ void GenericVector<T>::init_to_size(int size, const T& t) {
   }
 }
 
-// Return the object from an index.
 template <typename T>
-T& GenericVector<T>::get(int index) const {
+void GenericVector<T>::resize(int size, const T &t) {
+  init_to_size(size, t);
+}
+
+template <typename T>
+T &GenericVector<T>::operator[](int index) const {
   assert(index >= 0 && index < size_used_);
   return data_[index];
 }
 
 template <typename T>
-T& GenericVector<T>::operator[](int index) const {
-  assert(index >= 0 && index < size_used_);
-  return data_[index];
-}
-
-template <typename T>
-T& GenericVector<T>::back() const {
+T &GenericVector<T>::back() const {
   assert(size_used_ > 0);
   return data_[size_used_ - 1];
 }
@@ -795,7 +546,7 @@ T GenericVector<T>::pop_back() {
 
 // Return the object from an index.
 template <typename T>
-void GenericVector<T>::set(const T& t, int index) {
+void GenericVector<T>::set(const T &t, int index) {
   assert(index >= 0 && index < size_used_);
   data_[index] = t;
 }
@@ -804,7 +555,7 @@ void GenericVector<T>::set(const T& t, int index) {
 // space for the new elements and inserts the given element
 // at the specified index.
 template <typename T>
-void GenericVector<T>::insert(const T& t, int index) {
+void GenericVector<T>::insert(const T &t, int index) {
   assert(index >= 0 && index <= size_used_);
   if (size_reserved_ == size_used_) {
     double_the_size();
@@ -827,30 +578,17 @@ void GenericVector<T>::remove(int index) {
   size_used_--;
 }
 
-// Return true if the index is valindex
-template <typename T>
-T GenericVector<T>::contains_index(int index) const {
-  return index >= 0 && index < size_used_;
-}
-
 // Return the index of the T object.
 template <typename T>
-int GenericVector<T>::get_index(const T& object) const {
+int GenericVector<T>::get_index(const T &object) const {
   for (int i = 0; i < size_used_; ++i) {
-    assert(compare_cb_ != nullptr);
-    if (compare_cb_->Run(object, data_[i])) {
+    if (object == data_[i]) {
       return i;
     }
   }
   return -1;
 }
 
-// Return true if T is in the array
-template <typename T>
-bool GenericVector<T>::contains(const T& object) const {
-  return get_index(object) != -1;
-}
-
 // Add an element in the array
 template <typename T>
 int GenericVector<T>::push_back(T object) {
@@ -864,44 +602,21 @@ int GenericVector<T>::push_back(T object) {
 }
 
 template <typename T>
-int GenericVector<T>::push_back_new(const T& object) {
-  int index = get_index(object);
-  if (index >= 0) {
-    return index;
-  }
-  return push_back(object);
-}
-
-// Add an element in the array (front)
-template <typename T>
-int GenericVector<T>::push_front(const T& object) {
-  if (size_used_ == size_reserved_) {
-    double_the_size();
-  }
-  for (int i = size_used_; i > 0; --i) {
-    data_[i] = data_[i - 1];
-  }
-  data_[0] = object;
-  ++size_used_;
-  return 0;
-}
-
-template <typename T>
-void GenericVector<T>::operator+=(const T& t) {
+void GenericVector<T>::operator+=(const T &t) {
   push_back(t);
 }
 
 template <typename T>
-GenericVector<T>& GenericVector<T>::operator+=(const GenericVector& other) {
+GenericVector<T> &GenericVector<T>::operator+=(const GenericVector &other) {
   this->reserve(size_used_ + other.size_used_);
-  for (int i = 0; i < other.size(); ++i) {
+  for (unsigned i = 0; i < other.size(); ++i) {
     this->operator+=(other.data_[i]);
   }
   return *this;
 }
 
 template <typename T>
-GenericVector<T>& GenericVector<T>::operator=(const GenericVector& other) {
+GenericVector<T> &GenericVector<T>::operator=(const GenericVector &other) {
   if (&other != this) {
     this->truncate(0);
     this->operator+=(other);
@@ -914,17 +629,14 @@ template <typename T>
 void GenericVector<T>::clear() {
   if (size_reserved_ > 0 && clear_cb_ != nullptr) {
     for (int i = 0; i < size_used_; ++i) {
-      clear_cb_->Run(data_[i]);
+      clear_cb_(data_[i]);
     }
   }
   delete[] data_;
   data_ = nullptr;
   size_used_ = 0;
   size_reserved_ = 0;
-  delete clear_cb_;
   clear_cb_ = nullptr;
-  delete compare_cb_;
-  compare_cb_ = nullptr;
 }
 
 template <typename T>
@@ -935,8 +647,7 @@ void GenericVector<T>::delete_data_pointers() {
 }
 
 template <typename T>
-bool GenericVector<T>::write(
-    FILE* f, TessResultCallback2<bool, FILE*, T const&>* cb) const {
+bool GenericVector<T>::write(FILE *f, std::function<bool(FILE *, const T &)> cb) const {
   if (fwrite(&size_reserved_, sizeof(size_reserved_), 1, f) != 1) {
     return false;
   }
@@ -945,12 +656,10 @@ bool GenericVector<T>::write(
   }
   if (cb != nullptr) {
     for (int i = 0; i < size_used_; ++i) {
-      if (!cb->Run(f, data_[i])) {
-        delete cb;
+      if (!cb(f, data_[i])) {
         return false;
       }
     }
-    delete cb;
   } else {
     if (fwrite(data_, sizeof(T), size_used_, f) != unsigned_size()) {
       return false;
@@ -960,8 +669,7 @@ bool GenericVector<T>::write(
 }
 
 template <typename T>
-bool GenericVector<T>::read(
-    tesseract::TFile* f, TessResultCallback2<bool, tesseract::TFile*, T*>* cb) {
+bool GenericVector<T>::read(TFile *f, std::function<bool(TFile *, T *)> cb) {
   int32_t reserved;
   if (f->FReadEndian(&reserved, sizeof(reserved), 1) != 1) {
     return false;
@@ -972,14 +680,12 @@ bool GenericVector<T>::read(
   }
   if (cb != nullptr) {
     for (int i = 0; i < size_used_; ++i) {
-      if (!cb->Run(f, data_ + i)) {
-        delete cb;
+      if (!cb(f, data_ + i)) {
         return false;
       }
     }
-    delete cb;
   } else {
-    if (f->FReadEndian(data_, sizeof(T), size_used_) != size_used_) {
+    if (f->FReadEndian(data_, sizeof(T), size_used_) != static_cast<unsigned>(size_used_)) {
       return false;
     }
   }
@@ -989,7 +695,7 @@ bool GenericVector<T>::read(
 // Writes a vector of simple types to the given file. Assumes that bitwise
 // read/write of T will work. Returns false in case of error.
 template <typename T>
-bool GenericVector<T>::Serialize(FILE* fp) const {
+bool GenericVector<T>::Serialize(FILE *fp) const {
   if (fwrite(&size_used_, sizeof(size_used_), 1, fp) != 1) {
     return false;
   }
@@ -999,7 +705,7 @@ bool GenericVector<T>::Serialize(FILE* fp) const {
   return true;
 }
 template <typename T>
-bool GenericVector<T>::Serialize(tesseract::TFile* fp) const {
+bool GenericVector<T>::Serialize(TFile *fp) const {
   if (fp->FWrite(&size_used_, sizeof(size_used_), 1) != 1) {
     return false;
   }
@@ -1014,7 +720,7 @@ bool GenericVector<T>::Serialize(tesseract::TFile* fp) const {
 // Returns false in case of error.
 // If swap is true, assumes a big/little-endian swap is needed.
 template <typename T>
-bool GenericVector<T>::DeSerialize(bool swap, FILE* fp) {
+bool GenericVector<T>::DeSerialize(bool swap, FILE *fp) {
   uint32_t reserved;
   if (fread(&reserved, sizeof(reserved), 1, fp) != 1) {
     return false;
@@ -1040,7 +746,7 @@ bool GenericVector<T>::DeSerialize(bool swap, FILE* fp) {
   return true;
 }
 template <typename T>
-bool GenericVector<T>::DeSerialize(tesseract::TFile* fp) {
+bool GenericVector<T>::DeSerialize(TFile *fp) {
   uint32_t reserved;
   if (fp->FReadEndian(&reserved, sizeof(reserved), 1) != 1) {
     return false;
@@ -1055,20 +761,12 @@ bool GenericVector<T>::DeSerialize(tesseract::TFile* fp) {
   size_used_ = reserved;
   return fp->FReadEndian(data_, sizeof(T), size_used_) == size_used_;
 }
-template <typename T>
-bool GenericVector<T>::SkipDeSerialize(tesseract::TFile* fp) {
-  uint32_t reserved;
-  if (fp->FReadEndian(&reserved, sizeof(reserved), 1) != 1) {
-    return false;
-  }
-  return (uint32_t)fp->FRead(nullptr, sizeof(T), reserved) == reserved;
-}
 
 // Writes a vector of classes to the given file. Assumes the existence of
 // bool T::Serialize(FILE* fp) const that returns false in case of error.
 // Returns false in case of error.
 template <typename T>
-bool GenericVector<T>::SerializeClasses(FILE* fp) const {
+bool GenericVector<T>::SerializeClasses(FILE *fp) const {
   if (fwrite(&size_used_, sizeof(size_used_), 1, fp) != 1) {
     return false;
   }
@@ -1079,18 +777,6 @@ bool GenericVector<T>::SerializeClasses(FILE* fp) const {
   }
   return true;
 }
-template <typename T>
-bool GenericVector<T>::SerializeClasses(tesseract::TFile* fp) const {
-  if (fp->FWrite(&size_used_, sizeof(size_used_), 1) != 1) {
-    return false;
-  }
-  for (int i = 0; i < size_used_; ++i) {
-    if (!data_[i].Serialize(fp)) {
-      return false;
-    }
-  }
-  return true;
-}
 
 // Reads a vector of classes from the given file. Assumes the existence of
 // bool T::Deserialize(bool swap, FILE* fp) that returns false in case of
@@ -1098,25 +784,7 @@ bool GenericVector<T>::SerializeClasses(tesseract::TFile* fp) const {
 // this function. Returns false in case of error.
 // If swap is true, assumes a big/little-endian swap is needed.
 template <typename T>
-bool GenericVector<T>::DeSerializeClasses(bool swap, FILE* fp) {
-  int32_t reserved;
-  if (fread(&reserved, sizeof(reserved), 1, fp) != 1) {
-    return false;
-  }
-  if (swap) {
-    Reverse32(&reserved);
-  }
-  T empty;
-  init_to_size(reserved, empty);
-  for (int i = 0; i < reserved; ++i) {
-    if (!data_[i].DeSerialize(swap, fp)) {
-      return false;
-    }
-  }
-  return true;
-}
-template <typename T>
-bool GenericVector<T>::DeSerializeClasses(tesseract::TFile* fp) {
+bool GenericVector<T>::DeSerializeClasses(TFile *fp) {
   int32_t reserved;
   if (fp->FReadEndian(&reserved, sizeof(reserved), 1) != 1) {
     return false;
@@ -1130,101 +798,27 @@ bool GenericVector<T>::DeSerializeClasses(tesseract::TFile* fp) {
   }
   return true;
 }
-template <typename T>
-bool GenericVector<T>::SkipDeSerializeClasses(tesseract::TFile* fp) {
-  int32_t reserved;
-  if (fp->FReadEndian(&reserved, sizeof(reserved), 1) != 1) {
-    return false;
-  }
-  for (int i = 0; i < reserved; ++i) {
-    if (!T::SkipDeSerialize(fp)) {
-      return false;
-    }
-  }
-  return true;
-}
 
 // This method clear the current object, then, does a shallow copy of
 // its argument, and finally invalidates its argument.
 template <typename T>
-void GenericVector<T>::move(GenericVector<T>* from) {
+void GenericVector<T>::move(GenericVector<T> *from) {
   this->clear();
   this->data_ = from->data_;
   this->size_reserved_ = from->size_reserved_;
   this->size_used_ = from->size_used_;
-  this->compare_cb_ = from->compare_cb_;
   this->clear_cb_ = from->clear_cb_;
   from->data_ = nullptr;
   from->clear_cb_ = nullptr;
-  from->compare_cb_ = nullptr;
   from->size_used_ = 0;
   from->size_reserved_ = 0;
 }
 
 template <typename T>
 void GenericVector<T>::sort() {
-  sort(&tesseract::sort_cmp<T>);
+  sort(&sort_cmp<T>);
 }
 
-// Internal recursive version of choose_nth_item.
-// The algorithm used comes from "Algorithms" by Sedgewick:
-// http://books.google.com/books/about/Algorithms.html?id=idUdqdDXqnAC
-// The principle is to choose a random pivot, and move everything less than
-// the pivot to its left, and everything greater than the pivot to the end
-// of the array, then recurse on the part that contains the desired index, or
-// just return the answer if it is in the equal section in the middle.
-// The random pivot guarantees average linear time for the same reason that
-// n times vector::push_back takes linear time on average.
-// target_index, start and and end are all indices into the full array.
-// Seed is a seed for rand_r for thread safety purposes. Its value is
-// unimportant as the random numbers do not affect the result except
-// between equal answers.
-template <typename T>
-int GenericVector<T>::choose_nth_item(int target_index, int start, int end,
-                                      unsigned int* seed) {
-  // Number of elements to process.
-  int num_elements = end - start;
-  // Trivial cases.
-  if (num_elements <= 1) {
-    return start;
-  }
-  if (num_elements == 2) {
-    if (data_[start] < data_[start + 1]) {
-      return target_index > start ? start + 1 : start;
-    }
-    return target_index > start ? start : start + 1;
-  }
-// Place the pivot at start.
-#ifndef rand_r  // _MSC_VER, ANDROID
-  srand(*seed);
-#  define rand_r(seed) rand()
-#endif  // _MSC_VER
-  int pivot = rand_r(seed) % num_elements + start;
-  swap(pivot, start);
-  // The invariant condition here is that items [start, next_lesser) are less
-  // than the pivot (which is at index next_lesser) and items
-  // [prev_greater, end) are greater than the pivot, with items
-  // [next_lesser, prev_greater) being equal to the pivot.
-  int next_lesser = start;
-  int prev_greater = end;
-  for (int next_sample = start + 1; next_sample < prev_greater;) {
-    if (data_[next_sample] < data_[next_lesser]) {
-      swap(next_lesser++, next_sample++);
-    } else if (data_[next_sample] == data_[next_lesser]) {
-      ++next_sample;
-    } else {
-      swap(--prev_greater, next_sample);
-    }
-  }
-  // Now the invariant is set up, we recurse on just the section that contains
-  // the desired index.
-  if (target_index < next_lesser) {
-    return choose_nth_item(target_index, start, next_lesser, seed);
-  }
-  if (target_index < prev_greater) {
-    return next_lesser;  // In equal bracket.
-  }
-  return choose_nth_item(target_index, prev_greater, end, seed);
-}
+} // namespace tesseract
 
-#endif  // TESSERACT_CCUTIL_GENERICVECTOR_H_
+#endif // TESSERACT_CCUTIL_GENERICVECTOR_H_
