@@ -2,7 +2,6 @@
  * File:        tprintf.h
  * Description: Trace version of printf - portable between UX and NT
  * Author:      Phil Cheatle
- * Created:     Wed Jun 28 15:01:15 BST 1995
  *
  * (C) Copyright 1995, Hewlett-Packard Ltd.
  ** Licensed under the Apache License, Version 2.0 (the "License");
@@ -20,12 +19,29 @@
 #ifndef TESSERACT_CCUTIL_TPRINTF_H
 #define TESSERACT_CCUTIL_TPRINTF_H
 
-#include "platform.h"   // for TESS_API
+#include "params.h" // for BOOL_VAR_H
+#include <tesseract/export.h> // for TESS_API
+
+namespace tesseract {
+
+#if !defined(__GNUC__) && !defined(__attribute__)
+# define __attribute__(attr) // compiler without support for __attribute__
+#endif
+
+// Disable some log messages by setting log_level > 0.
+extern TESS_API INT_VAR_H(log_level);
 
 // Main logging function.
-extern TESS_API void tprintf(  // Trace printf
-    const char *format, ...);  // Message
+extern TESS_API void tprintf( // Trace printf
+    const char *format, ...)  // Message
+    __attribute__((format(printf, 1, 2)));
+
 /* willus mod */
 extern TESS_API void tprintf_set_debugfile(const char *filename);
 
-#endif  // define TESSERACT_CCUTIL_TPRINTF_H
+
+} // namespace tesseract
+
+#undef __attribute__
+
+#endif // define TESSERACT_CCUTIL_TPRINTF_H
